/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.ui.impl;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.JComponent;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import org.netbeans.api.settings.ConvertAsProperties;
import it.tidalwave.actor.spi.ActorActivator;
import it.tidalwave.actor.spi.ActorGroupActivator;
import it.tidalwave.integritychecker.ui.IntegrityCheckerView;
import it.tidalwave.integritychecker.ui.impl.action.ScanActionEnablerActor;
import it.tidalwave.netbeans.util.Locator;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 * 
 * @stereotype View
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ConvertAsProperties(dtd = "-//it.tidalwave.integritychecker.ui.impl//IntegrityCheckerView//EN", autostore = false)
@Slf4j
public final class IntegrityCheckerViewTopComponent extends TopComponent 
  {
    private static IntegrityCheckerViewTopComponent instance;

    private static final String PREFERRED_ID = "IntegrityCheckerViewTopComponent";
    
    private final IntegrityCheckerViewBuilder viewBuilder = Locator.find(IntegrityCheckerViewBuilder.class);
    
    private final IntegrityCheckerView view = (IntegrityCheckerView)(viewBuilder.createGui());
    
    private final ActorGroupActivator activator = new IntegrityCheckerUIActivator();
    
    public IntegrityCheckerViewTopComponent() 
      {
        initComponents();
        pnContents.setLayout(new GridBagLayout());
        final GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 1;
        c.gridheight = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.CENTER;
        pnContents.setVisible(true);
        pnContents.add((JComponent)view, c);
        setName(NbBundle.getMessage(IntegrityCheckerViewTopComponent.class, "CTL_IntegrityCheckerViewTopComponent"));
//        setToolTipText(NbBundle.getMessage(IntegrityCheckerViewTopComponent.class, "HINT_IntegrityCheckerViewTopComponent"));
        putClientProperty(TopComponent.PROP_CLOSING_DISABLED, Boolean.TRUE);
        putClientProperty(TopComponent.PROP_DRAGGING_DISABLED, Boolean.TRUE);
        putClientProperty(TopComponent.PROP_MAXIMIZATION_DISABLED, Boolean.TRUE);
        putClientProperty(TopComponent.PROP_UNDOCKING_DISABLED, Boolean.TRUE);
      }

    @Override
    public int getPersistenceType()
      {
        return TopComponent.PERSISTENCE_ALWAYS;
      }

    @Override
    public void componentOpened() 
      {
        activator.activate();
      }

    @Override
    public void componentClosed() 
      {
        activator.deactivate();
      }

    void writeProperties (java.util.Properties p)
      {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        // TODO store your settings
      }

    Object readProperties (java.util.Properties p)
      {
        if (instance == null) 
          {
            instance = this;
          }
        
        instance.readPropertiesImpl(p);
        return instance;
      }

    private void readPropertiesImpl(java.util.Properties p) 
      {
        String version = p.getProperty("version");
        // TODO read your settings according to their version
      }

    @Override
    protected String preferredID() 
      {
        return PREFERRED_ID;
      }
    
    /** 
     * 
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always regenerated by the Form Editor.
     * 
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        pnContents = new javax.swing.JPanel();

        setName("Form"); // NOI18N

        pnContents.setName("pnContents"); // NOI18N
        pnContents.setOpaque(false);

        javax.swing.GroupLayout pnContentsLayout = new javax.swing.GroupLayout(pnContents);
        pnContents.setLayout(pnContentsLayout);
        pnContentsLayout.setHorizontalGroup(
            pnContentsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 428, Short.MAX_VALUE)
        );
        pnContentsLayout.setVerticalGroup(
            pnContentsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 325, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(pnContents, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(pnContents, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel pnContents;
    // End of variables declaration//GEN-END:variables
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link #findInstance}.
     */
    public static synchronized IntegrityCheckerViewTopComponent getDefault() 
      {
        if (instance == null) 
          {
            instance = new IntegrityCheckerViewTopComponent();
          }
        
        return instance;
      }

    /**
     * Obtain the IntegrityCheckerViewTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized IntegrityCheckerViewTopComponent findInstance() 
      {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        
        if (win == null) 
          {
            log.warn("Cannot find " + PREFERRED_ID + " component. It will not be located properly in the window system.");
            return getDefault();
          }
        
        if (win instanceof IntegrityCheckerViewTopComponent) 
          {
            return (IntegrityCheckerViewTopComponent) win;
          }
        
        log.warn( "There seem to be multiple components with the '" + PREFERRED_ID
                + "' ID. That is a potential source of errors and unexpected behavior.");
        
        return getDefault();
      }
  }
